//=======================================================//
// Maarten L. Buis, version 1.0.2, 21 Okt  2010          //
//                  version 1.0.0  09 Sept 2010          //
//                                                       //
// Master file for reproduding the analyses of           //
// "The Consequences of Unobserved Heterogeneity in a    //
// Sequential Logit Model"                               //
//                                                       //
// To run this, all the files in the .zip package should //
// be placed in the same directory, and the line         //
// starting with -global h "c:\mijn documenten\...       //
// should be changed to point at the directory.          //
//                                                       //
// After that you can just do this .do file              //
//                                                       //
// Warning: the .zip file contains a copy the -seqlogit- //
// package that is frozen at version 1.1.13. The purpose //
// of these files is to be able to reproduce my analysis.//
// If you intend to use this package for your own        //
// research you should get the latest version by typing  //
// in Stata:                                             //
// ssc install seqlogit, replace                         //
//                                                       //
//=======================================================//

// Some of the graphs require the -grc1leg- package by Vince Wiggins
// If that is not installed, you can get it by removing the 
// the comment "//" in the following line:
// net install grc1leg, from(http://www.stata.com/users/vwiggins)

version 11.1
clear all
discard
set more off
set memory 60m

// define the directory where all do files and the
// data are stored. You should only have to change
// this line to be able to run the analysis. 
cd "c:\mijn documenten\projecten\rssm\1.0.2"

// change the amount of unobserved heterogeneity
do "sd"                 // estimate the scenarios
do "graph_sd"           // extract information from the scenarios and 
                        // graph the results
do "table_null"         // collect results for the baseline model (no 
                        // unobserved heterogeneity) in a table
do "table_sd"           // collect information from scenarios in matrix
						
// let the amount of unobserved heterogeneity change over transitions
do "sd_tr"              // estimate the scenarios
do "graph_sd_tr"        // extract information from the scenarios and
                        // graph the results
do "table_sd_tr"        // collect information from scenarios in matrix

// let the amount of unobserved heterogeneity change over cohorts						
do "sd_coh"             // estimate the scenarios
do "graph_sd_coh"       // extract information from the scenarios and
                        // graph the results
do "table_sd_coh"       // collect information from scenarios in matrix

// let the correlation between the unobserved variable and father's
// education during the first transition change						
do "rho"                // estimate the scenarios
do "graph_rho"          // extract information from the scenarios and
                        // graph the results
do "table_rho"          // collect information from scenarios in matrix

// let the distribution of the unobserved variable change						
do "dist"               // estimate the scenarios
do "graph_dist"         // extract information from the scenarios and
                        // graph the result
do "table_dist"          // collect information from scenarios in matrix

// collect and display matrices made in the table_*.do files
matrix par = par1 \ par2 \ par3 \ par4 \par5
matrix unrob = un_rob1 \ un_rob2 \ un_rob3 \ un_rob4 \ un_rob5

matlist par0, format(%9.3f)
matlist par, format(%9.3f)
matlist unrob, format(%9.3f)

exit
