//===============================================================================
// This do file looks at the results from the scenarios investigating the effect
// of changes in the amount of unobserved heterogeneity over transitions and
// stores this in a matrix par5 and un_rob5
//
// This do file requires that dist.do and graph_dist.do has been run
//
// I will recover from this model the following elements:
//
// ==> the change in effect of father's education between transitions 1 and 2 and
//     transition 2 and 3
//
// ==> The change in effect of father's educaiton for a decade increase in cohort
//
// ==> One sided p-values of the test of the null-hypothesis that these changes 
//     are 0 or larger, i.e. we find negative trends if we reject the null 
//
// Tricks used for getting one-sided test are explained here:
// http://www.stata.com/support/faqs/stat/oneside.html
//===============================================================================


// create matrix in which results will be stored and row and column names
matrix par5 = J(5,4, .)
matrix colname par5 = b:min b:max p:min p:max
matrix rowname par5 = dist:coh_1 dist:coh_2 dist:coh_3 dist:trans_2_1 dist:trans_3_2

matrix un_rob5 = J(5, 11, .)
matrix rowname un_rob5 = dist:coh_1 dist:coh_2 dist:coh_3 dist:trans_2_1 dist:trans_3_2


// create temprary names for scalars
tempname sc_b_min sc_b_max sc_p_min sc_p_max

// ===============================================================================
// change over cohort
// ===============================================================================
// collect ranges of parameters
forvalues i = 1/3 {
	sum b_coh_`i', meanonly
	scalar `sc_b_min' = r(min)
	scalar `sc_b_max' = r(max)
	sum p_coh_`i', meanonly
	scalar `sc_p_min' = r(min)
	scalar `sc_p_max' = r(max)
	matrix par5[`i', 1] = `sc_b_min', `sc_b_max' , `sc_p_min', `sc_p_max'        // fill in the table
}

// collect non-robust scenarios
local j = 1
forvalues i = 1/7 {
	if p_coh_1[`i'] > .05 {
		matrix un_rob5[1,`j'] = dist[`i']
		local j = `j' + 1
	}
}
local j = 1
forvalues i = 1/7 {
	if p_coh_2[`i'] > .05 {
		matrix un_rob5[2,`j'] = dist[`i']
		local j = `j' + 1
	}
}
local j = 1
forvalues i = 1/7 {
	if p_coh_3[`i'] > .05 {
		matrix un_rob5[3,`j'] = dist[`i']
		local j = `j' + 1
	}
}



// ===============================================================================
// change over transitions
// ===============================================================================
// collect ranges of parameters
sum b_tr_1, meanonly
scalar `sc_b_min' = r(min)
scalar `sc_b_max' = r(max)
sum p_tr_1, meanonly
scalar `sc_p_min' = r(min)
scalar `sc_p_max' = r(max)
matrix par5[4, 1] = `sc_b_min', `sc_b_max' , `sc_p_min', `sc_p_max'        // fill in the table

sum b_tr_2, meanonly
scalar `sc_b_min' = r(min)
scalar `sc_b_max' = r(max)
sum p_tr_2, meanonly
scalar `sc_p_min' = r(min)
scalar `sc_p_max' = r(max)
matrix par5[5, 1] = `sc_b_min', `sc_b_max' , `sc_p_min', `sc_p_max'        // fill in the table

// collect non-robust scenarios
local j = 1
forvalues i = 1/7 {
	if p_tr_1[`i'] < .05 {
		matrix un_rob5[4,`j'] = dist[`i']
		local j = `j' + 1
	}
}
local j = 1
forvalues i = 1/7 {
	if p_tr_2[`i'] > .05 {
		matrix un_rob5[5,`j'] = dist[`i']
		local j = `j' + 1
	}
}

exit
