//===============================================================================
// This do file looks at the results from the baseline model (no unobserved 
// heterogeneity) and stores this in a matrix par0
//
// This do file requires that sd.do has been run
//
// I will recover from this model the following elements:
//
// ==> the change in effect of father's education between transitions 1 and 2 and
//     transition 2 and 3
//
// ==> The change in effect of father's educaiton for a decade increase in cohort
//
// ==> One sided p-values of the test of the null-hypothesis that these changes 
//     are 0 or larger, i.e. we find negative trends if we reject the null 
//
// Tricks used for getting one-sided test are explained here:
// http://www.stata.com/support/faqs/stat/oneside.html
//===============================================================================


// open baseline model 
est restore null

// create matrix in which results will be stored and row and column names
matrix par0 = J(5,4, .)
matrix colname par0 = b:0 b:2,5 p:0 p:2,5
matrix rowname par0 = null:coh_1 null:coh_2 null:coh_3 null:trans_2_1 null:trans_3_2

// create temprary names for scalars
tempname sc_b_0 sc_p_0 sc_b_25 sc_p_25

// ===============================================================================
// change over cohort
// ===============================================================================
forvalues i = 1/3 {
	est restore null
	test [#`i']_b[_fed_X_coh] = 0
	local sign = sign([#`i']_b[_fed_X_coh])
	scalar `sc_p_0' =  normal(`sign'*sqrt(r(chi2)))                    // one sided p-value
	scalar `sc_b_0' = (exp([#`i']_b[_fed_X_coh])-1)*100                // % change in odds for a decade change in cohort
	
	est use "sens", number(5)
	test [#`i']_b[_fed_X_coh] = 0
	local sign = sign([#`i']_b[_fed_X_coh])
	scalar `sc_p_25' =  normal(`sign'*sqrt(r(chi2)))                   // one sided p-value
	scalar `sc_b_25' = (exp([#`i']_b[_fed_X_coh])-1)*100               // % change in odds for a decade change in cohort
	matrix par0[`i', 1] = `sc_b_0', `sc_b_25'  , `sc_p_0', `sc_p_25'   // fill in the table
}


// ===============================================================================
// change over transitions
// ===============================================================================

// coh = 6 for the cohort born in 1960, so
// [#]_b[fed] + [#]_b[_fed_X_coh]*6
// is the effect of fed for that cohort in equation #
local diff1 "[#2]_b[fed] + [#2]_b[_fed_X_coh]*6 - [#1]_b[fed] - [#1]_b[_fed_X_coh]*6"
local diff2 "[#3]_b[fed] + [#3]_b[_fed_X_coh]*6 - [#2]_b[fed] - [#2]_b[_fed_X_coh]*6"

est restore null
test `diff1' = 0
local sign = sign(`diff1')
scalar `sc_p_0' =  normal(`sign'*sqrt(r(chi2)))               // one-sided p-value
scalar `sc_b_0' = (exp(`diff1')-1)*100                        // % change in odds ratio from trans. 1 to trans. 2

est use "sens", number(5)
test `diff1' = 0
local sign = sign(`diff1')
scalar `sc_p_25' =  normal(`sign'*sqrt(r(chi2)))              // one-sided p-value
scalar `sc_b_25' = (exp(`diff1')-1)*100                       // % change in odds ratio from trans. 1 to trans. 2

matrix par0[4,1] = `sc_b_0', `sc_b_25' , `sc_p_0', `sc_p_25'  // fill in the table

est restore null
test `diff2' = 0
local sign = sign(`diff2')
scalar `sc_p_0' =  normal(`sign'*sqrt(r(chi2)))               // one-sided p-value
scalar `sc_b_0' = (exp(`diff2')-1)*100                        // % change in odds ratio from trans. 2 to trans. 3

est use "sens", number(5)
test `diff2' = 0
local sign = sign(`diff2')
scalar `sc_p_25' =  normal(`sign'*sqrt(r(chi2)))              // one-sided p-value
scalar `sc_b_25' = (exp(`diff2')-1)*100                       // % change in odds ratio from trans. 2 to trans. 3
matrix par0[5,1] = `sc_b_0', `sc_b_25' , `sc_p_0', `sc_p_25'  // fill in the table

exit
